<?php

class PNOptions
{
	public $options;
	public $pnOptionsPost;
	private $settings_saved;
	private $settings_error;
	
	public function __construct()
	{
		$this->options = array();
		$this->pnOptionsPost = new PNOptionsPost();
		$this->settings_saved = 0;
		$this->settings_error = 0;
	}
	
	public function set_post_id($post_id)
	{
		$this->pnOptionsPost->post_id = $post_id;
	}
	
	/**
	 * Returns post option if it's specified and enabled, otherwise returns 
	 * value for global option
	 * 
	 * @param string $name - name of the option
	 * @return mixed - value for the option, false if doesn't exist.
	 */
	public function __get($name)
	{
		$this->options_load();
		$options = $this->options;
		$this->pnOptionsPost->options_post_load();
		$options_post = $this->pnOptionsPost->options_post;
		if(
			isset($options_post[$name . '_default']) && 
			!$options_post[$name . '_default']
		)
			return $options_post[$name];
		else
			return (isset($options[$name]) ? $options[$name] : false);
	}
	
	/**
	 * Returns value for global option
	 * 
	 * @param string $name - name of the option
	 * @return mixed - value for the option, false if doesn't exist.
	 */
	public function get_option($name)
	{
		$this->options_load();
		$options = $this->options;
		return (isset($options[$name]) ? $options[$name] : false);
	}
	
	/**
	 * Returns value for post option
	 * 
	 * @param string $name - name of the option
	 * @return mixed - value for the option, false if doesn't exist.
	 */
	public function get_option_post($name)
	{
		$this->pnOptionsPost->options_post_load();
		$options_post = $this->pnOptionsPost->options_post;
		return (isset($options_post[$name]) ? $options_post[$name] : false);
	}
	
	public function handle_wp()
	{
		//action must be registered with priority higher than 10.
		add_action('wp_loaded', array($this, 'options_save'), 11);
		add_action('admin_menu', array($this, 'options_panel'));
		add_action('admin_notices', array($this, 'options_panel_notices'));
		add_action('wp_ajax_pn_get_font_subsets', array($this, 'get_font_subsets_ajax'));
	}
	
	function get_font_subsets_ajax()
	{
		$result = array();
		if($_POST["font"]!="")
		{
			$subsets = '';
			$fontExplode = explode(":", $_POST["font"]);
			$subsets_array = PNFonts::get_google_font_subsets($fontExplode[0]);
			
			if($subsets_array!==false)
				foreach($subsets_array as $subset)
					$subsets .= '<option value="' . $subset . '">' . $subset . '</option>';

			$result = array();
			$result['font_subsets'] = $subsets;
		}
		echo @json_encode($result);
		die;
	}
	
	public function options_save()
	{
		if(isset($_POST['pnoptions']))
		{
			$nonce = $_POST['pnoptions_wpnonce'];
			if(!wp_verify_nonce($nonce, 'pnoptions_save')) 
				die('Nonce is invalid');
			
			$options = array(
				'post_types'							=> (isset($_POST['post_types']) ? $_POST['post_types'] : ''),
				'processing_method'						=> (isset($_POST['processing_method']) ? $_POST['processing_method'] : ''),
				'progress_bar_algorithm'				=> (isset($_POST['progress_bar_algorithm']) ? $_POST['progress_bar_algorithm'] : ''),
				'enable_navigation_bar'					=> (isset($_POST['enable_navigation_bar']) ? (int)$_POST['enable_navigation_bar'] : ''),
				'progress_bar_height'					=> (isset($_POST['progress_bar_height']) && (int)$_POST['progress_bar_height']>0 ? (int)$_POST['progress_bar_height'] : 3),				
				'overlap_site_content'					=> (isset($_POST['overlap_site_content']) ? (int)$_POST['overlap_site_content'] : ''),
				'bar_ending'							=> (isset($_POST['bar_ending']) ? $_POST['bar_ending'] : ''),
				'enable_sections'						=> (isset($_POST['enable_sections']) ? (int)$_POST['enable_sections'] : ''),
				'show_section_marks'					=> (isset($_POST['show_section_marks']) ? $_POST['show_section_marks'] : ''),
				'section_selectors'						=> (isset($_POST['section_selectors']) ? $_POST['section_selectors'] : ''),
				'section_offset'						=> (isset($_POST['section_offset']) && (int)$_POST['section_offset']>=0 ? (int)$_POST['section_offset'] : 50),
				'section_title_selectors'				=> (isset($_POST['section_title_selectors']) ? $_POST['section_title_selectors'] : ''),
				'section_img_selectors'					=> (isset($_POST['section_img_selectors']) ? $_POST['section_img_selectors'] : ''),
				'sections_container_selectors'			=> (isset($_POST['sections_container_selectors']) ? $_POST['sections_container_selectors'] : ''),
				'navigation_menu'						=> (isset($_POST['navigation_menu']) ? $_POST['navigation_menu'] : ''),
				'section_tooltip'						=> (isset($_POST['section_tooltip']) ? (int)$_POST['section_tooltip'] : ''),
				'section_tooltip_image'					=> (isset($_POST['section_tooltip_image']) ? (int)$_POST['section_tooltip_image'] : ''),
				'section_tooltip_description'			=> (isset($_POST['section_tooltip_description']) ? (int)$_POST['section_tooltip_description'] : ''),
				'section_tooltip_description_length'	=> (isset($_POST['section_tooltip_description_length']) && (int)$_POST['section_tooltip_description_length']>0 ? (int)$_POST['section_tooltip_description_length'] : 20),				
				'section_tooltip_pagination'			=> (isset($_POST['section_tooltip_pagination']) ? (int)$_POST['section_tooltip_pagination'] : ''),
				'progress_bar_color_1'					=> (isset($_POST['progress_bar_color_1']) && strlen($_POST['progress_bar_color_1']) ? $_POST['progress_bar_color_1'] : '52b5e7'),
				'progress_bar_color_2'					=> (isset($_POST['progress_bar_color_2']) && strlen($_POST['progress_bar_color_2']) ? $_POST['progress_bar_color_2'] : 'e9e9e9'),
				'progress_bar_color_3'					=> (isset($_POST['progress_bar_color_3']) && strlen($_POST['progress_bar_color_3']) ? $_POST['progress_bar_color_3'] : 'e9e9e9'),
				'section_tooltip_background'			=> (isset($_POST['section_tooltip_background']) && strlen($_POST['section_tooltip_background']) ? $_POST['section_tooltip_background'] : '223138'),
				'header_font_custom'					=> (isset($_POST['header_font_custom']) ? $_POST['header_font_custom'] : ''),
				'header_font'							=> (isset($_POST['header_font']) ? $_POST['header_font'] : ''),
				'header_font_subset'					=> (isset($_POST['header_font_subset']) ? $_POST['header_font_subset'] : ''),
				'text_font_custom'						=> (isset($_POST['text_font_custom']) ? $_POST['text_font_custom'] : ''),
				'text_font'								=> (isset($_POST['text_font']) ? $_POST['text_font'] : ''),
				'text_font_subset'						=> (isset($_POST['text_font_subset']) ? $_POST['text_font_subset'] : ''),
				'text_font'								=> (isset($_POST['text_font']) ? $_POST['text_font'] : ''),
				'custom_css'							=> (isset($_POST['custom_css']) ? $_POST['custom_css'] : ''),
			);

			if(update_option('pn_options', $options))
				$this->settings_saved = 1;
		}
	}
	
	public function options_load($force = false)
	{
		if(count($this->options) && !$force)
			return;
		$options = get_option('pn_options');
		$this->options = (!$options ? $this->options_defaults() : $options);
	}
	
	public function options_defaults()
	{
		$options = array(
			'post_types'							=> array('post', 'page'),
			'enable_navigation_bar'					=> 1,
			'processing_method'						=> 'output_buffering',
			'progress_bar_algorithm'				=> 'algorithm_1',
			'bar_ending'							=> 'end_of_content',
			'overlap_site_content'					=> 0,
			'progress_bar_height'					=> 3,
			'enable_sections'						=> 1,
			'show_section_marks'					=> 'always',
			'section_offset'						=> 50,
			'section_selectors'						=> 'h1,h2,h3',
			'section_title_selectors'				=> '',
			'section_img_selectors'					=> '',
			'sections_container_selectors'			=> '',
			'navigation_menu'						=> 'always_visible',
			'section_tooltip'						=> 1,
			'section_tooltip_image'					=> 1,
			'section_tooltip_description'			=> 1,
			'section_tooltip_description_length'	=> 20,
			'section_tooltip_pagination'			=> 1,
			'progress_bar_color_1'					=> '52b5e7',
			'progress_bar_color_2'					=> 'e9e9e9',
			'progress_bar_color_3'					=> 'e9e9e9',
			'section_tooltip_background'			=> '223138',
			'header_font_custom'					=> '',
			'header_font'							=> '',
			'header_font_subset'					=> array(),
			'text_font_custom'						=> '',
			'text_font'								=> '',
			'text_font_subset'						=> array(),
			'custom_css'							=> '',
		);
		update_option('pn_options', $options);
		return $options;
	}
	
	public function options_panel()
	{
		//register options page
		add_options_page('Site Content Navigator', 'Site Content Navigator', 'manage_options', 'site-content-navigator.php', array($this, 'options_panel_html'));
	}
	
	public function options_panel_html()
	{
		$this->options_load(true);
		$post_types = get_post_types();
		$google_fonts = PNFonts::get_google_font();
		echo include(PN_TEMPLATE_PATH . 'options.php');
	}
	
	public function options_panel_notices()
	{
		if($this->settings_saved)
		{
			add_settings_error(
				'pn_settings_saveds',
				esc_attr('settings_updated'),
				__('Settings saved.', 'site_content_navigator'),
				'updated'
			);
			$this->settings_saved = 0;
		}
		
		if($this->settings_error)
		{
			add_settings_error(
				'pn_settings_saveds',
				esc_attr('settings_updated'),
				__('Settings couldn\'t be saved.', 'site_content_navigator')
			);
			$this->settings_error = 0;
		}
	}
	
	public function is_post_type_enabled($post_type)
	{
		$post_types = $this->get_option('post_types');
		if(!is_array($post_types))
			return false;
		else
		{
			if(in_array($post_type, $post_types))
				return true;
			else
				return false;
		}
	}
	
}
