<?php

class PNOptionsPost
{
	public $post_id;
	public $options_post;
	public $post_types;
	
	public function __construct()
	{
		$this->options_post = array();
		$this->post_types = array();
	}
	
	public function handle_wp()
	{
		add_action('add_meta_boxes', array($this, 'register_meta_box'), 10, 2);
		add_action('save_post', array($this, 'options_post_save'));
		add_action('wp_ajax_pn_add_post_section', array($this, 'add_post_section_ajax'));
		add_action('wp_ajax_pn_find_post_sections', array($this, 'find_post_sections_ajax'));
		add_action('wp_ajax_pn_reset_post_sections', array($this, 'reset_post_sections_ajax'));		
	}
	
	/**
	 * Returns value for post option
	 * 
	 * @param string $name - name of the option
	 * @return mixed - value for the option, false if doesn't exist.
	 */
	public function get_option_post($name)
	{
	   $this->options_post_load();
	   $options_post = $this->options_post;
	   return (isset($options_post[$name]) ? $options_post[$name] : false);
	}
	
	public function options_post_load($force = false)
	{
		if(count($this->options_post))
			return;
		
		if($this->post_id || $this->post_id = get_the_ID())
			$this->options_post = get_post_meta($this->post_id, 'pn_options_post', true);
		else
			return false;
	}
	
	public function options_post_save($post_id)
	{
		if(isset($_POST['pnoptions_post']) && $_POST['pnoptions_post'])
		{
			$nonce = $_POST['pnoptions_post_wpnonce'];
			if(!wp_verify_nonce($nonce, 'pnoptions_post_save')) 
				die('Nonce is invalid');
			
			$options_post = array(
				'enable_navigation_bar_default'				=> (int)$_POST['enable_navigation_bar_default'],
				'enable_navigation_bar'						=> (int)$_POST['enable_navigation_bar'],
				'section_selectors_default'					=> (int)$_POST['section_selectors_default'],
				'section_selectors'							=> $_POST['section_selectors'],
				'section_title_selectors_default'			=> (int)$_POST['section_title_selectors_default'],
				'section_title_selectors'					=> $_POST['section_title_selectors'],
				'section_img_selectors_default'				=> (int)$_POST['section_img_selectors_default'],
				'section_img_selectors'						=> $_POST['section_img_selectors'],
				'post_sections_default'						=> (int)$_POST['post_sections_default'],
			);
			
			//prepare information on post sections settings
			$i = 0;
			$post_sections = array();
			if(isset($_POST['section_title']) && $count = count($_POST['section_title']))
			{
				for($i=0; $i<$count; $i++)
				{
					if($_POST['section_class'][$i]=='')
						continue;
					
					$post_sections[] = array(
						'title'			=> $_POST['section_title'][$i],
						'class'			=> $_POST['section_class'][$i],
						'description'	=> $_POST['section_description'][$i],
						'img'			=> $_POST['section_img'][$i],
						'img_id'		=> $_POST['section_img_id'][$i],
						'manual'		=> (int)$_POST['section_manual'][$i],
					);
				}
			}
			$options_post['post_sections'] = $post_sections;
			
			update_post_meta($post_id, 'pn_options_post', $options_post);
		}
	}
	
	public function register_meta_box($post_type)
	{
		if(in_array($post_type, $this->post_types))
		{
			add_meta_box( 
				'pn-options-post',
				'Site Content Navigator',
				array($this, 'meta_box_html')
			);
		}
	}
	
	public function meta_box_html($post_id)
	{
		$this->options_post_load(true);
		echo include(PN_TEMPLATE_PATH . 'options_post.php');
	}
	
	function add_post_section_ajax()
	{
		$section_index = $_POST['section_index'];
		
		$result = array();
		$result['post_section_empty_html'] = $this->post_section_empty_html($section_index);
		echo @json_encode($result);
		die;
	}
	
	public function find_post_sections_ajax()
	{
		$post_content = (isset($_POST['post_content']) ? stripslashes($_POST['post_content']) : '');
		$post_id = (int)$_POST['post_id'];
		$this->post_id = $post_id;
		$post_sections = (isset($_POST['post_sections']) ? stripslashes_deep($_POST['post_sections']) : '');
		$post_options = $_POST['post_options'];
		$section_index = $_POST['section_index'];
		$config = array(
			'post_id'			=> (int)$_POST['post_id'],
			'post_sections'		=> $post_sections,
			'post_options'		=> $post_options,
		);
		$pageSections = new PageSections();
		
		$post_content_changed = html_entity_decode($pageSections->process_content($post_content, $config));
		$page_sections = $pageSections->sections;
		$sections_html = '';
		$sections_html = $this->post_sections_html($page_sections, false, $section_index);
		
		$result = array();
		$result['page_sections_html'] = $sections_html;
		$result['post_content'] = $post_content_changed;
		echo @json_encode($result);
		die;
	}
	
	public function reset_post_sections_ajax()
	{
		$this->post_id = (int)$_POST['post_id'];
		
		$this->options_post_load();
		$sections_html = (isset($this->options_post['post_sections']) ? $this->post_sections_html($this->options_post['post_sections']) : '');		
		$result = array();
		$result['page_sections_html'] = $sections_html;
		echo @json_encode($result);
		die;
	}
	
	function post_sections_html($post_sections, $hide=true, $last_index=0)
	{
		return include(PN_TEMPLATE_PATH . 'section.php');
	}
	
	function post_section_empty_html($index)
	{
		return include(PN_TEMPLATE_PATH . 'section_empty.php');
	}
}

