<?php

class PageSections
{
	public $sections;
	public $pnOptions;
	
	public function __construct()
	{
		$this->sections = array();
		$this->pnOptions = new PNOptions();
	}
	
	public function process_content($content, $config = array())
	{
		if(!strlen($content))
			return $content;
		
		//PREPARE VARIABLES
		extract(array(
			'post_id'			=> isset($config['post_id']) ?  $config['post_id'] : null,
			'post_sections'		=> isset($config['post_sections']) ?  $config['post_sections'] : null,
			'post_options'		=> isset($config['post_options']) ?  $config['post_options'] : null,
		));
		if($post_id)
			$this->pnOptions->set_post_id($post_id);
		$post_sections_data = ($post_sections ? $post_sections : $this->pnOptions->post_sections);
		
		$section_selectors = (isset($post_options['section_selectors_default']) ? $this->pnOptions->get_option('section_selectors') : (isset($post_options['section_selectors']) ? $post_options['section_selectors'] : $this->pnOptions->section_selectors));
		$section_selectors = explode(',', $section_selectors);
		$section_selectors = array_map('trim', $section_selectors);
		
		$section_title_selectors = (isset($post_options['section_title_selectors_default']) ? $this->pnOptions->get_option('section_title_selectors') : (isset($post_options['section_title_selectors']) ? $post_options['section_title_selectors'] : $this->pnOptions->section_title_selectors));
		if(strlen(preg_replace('/\s+/', '', $section_title_selectors)))
		{	
			$section_title_selectors = explode(',', $section_title_selectors);
			$section_title_selectors = array_map('trim', $section_title_selectors);
		}
		else
			$section_title_selectors = $section_selectors;
		
		$section_img_selectors = (isset($post_options['section_img_selectors_default']) ? $this->pnOptions->get_option('section_img_selectors') : (isset($post_options['section_img_selectors']) ? $post_options['section_img_selectors'] : $this->pnOptions->section_img_selectors));
		$section_img_selectors = explode(',', $section_img_selectors);
		$section_img_selectors = array_map('trim', $section_img_selectors);
		
		$section_tooltip_description_length = $this->pnOptions->section_tooltip_description_length;

		//LOAD DOCUMENT/FIND SECTIONS/DELETE UNWANTED OLD SECTIONS
		libxml_use_internal_errors(true);
		$document = new \DOMDocument();

		if(!$document->loadHTML('<?xml encoding="UTF-8">' . $content))		
			return $content;
		
		$xpath = new \DOMXPath($document);
		
		//findDomNodeList will use strict comparison, may not work for old sections
		$sections_old = $this->find_sections_old($xpath);
		$sections = $this->findDomNodeList($xpath, $section_selectors);
		$sections_titles = $this->findDomNodeList($xpath, $section_title_selectors);
		$sections_images = $this->findDomNodeList($xpath, $section_img_selectors);

		$this->remove_unwanted_sections($sections, $sections_old);
		unset($sections_old);
		
		//assign titles/images to the sections
		$sections_arr = array();
		for ($i=0, $count_i=count($sections); $i<$count_i; $i++)
		{
			$sections_arr[$i]['section'] = $sections[$i];
			if(count($sections_titles))
			{
				foreach($sections_titles as $index=>$sections_title)
				{
					if(strpos($sections_title->getNodePath(), $sections_arr[$i]['section']->getNodePath())!==false)
					{
						$sections_arr[$i]['title'] = $sections_title;
						unset($sections_titles[$index]);
						break;
					}
				}
			}
			if(count($sections_images))
			{
				foreach($sections_images as $index=>$sections_image)
				{
					if(strpos($sections_image->getNodePath(), $sections_arr[$i]['section']->getNodePath())!==false)
					{
						$sections_arr[$i]['image'] = $sections_image;
						unset($sections_images[$index]);
						break;
					}
				}
			}
		}
		
		//CONFIGURE NEW SECTIONS
		$sections_count = count($this->sections);
		for ($i=0, $count=count($sections_arr); $i<$count; $i++)
		{
			$section_title = '';
			$section_description = '';
			if(isset($sections_arr[$i]['title']))
			{
				$section_title = $sections_arr[$i]['title']->nodeValue;
				if($sections_arr[$i]['title']->nextSibling!=NULL)
				{
					if(preg_replace('/\s+/', '', $sections_arr[$i]['title']->nextSibling->nodeValue)!='')
						$section_description = $sections_arr[$i]['title']->nextSibling->nodeValue;
					elseif($sections_arr[$i]['title']->nextSibling->nextSibling!=NULL && preg_replace('/\s+/', '', $sections_arr[$i]['title']->nextSibling->nextSibling->nodeValue)!="")
						$section_description = $sections_arr[$i]['title']->nextSibling->nextSibling->nodeValue;
					$section_description = wp_trim_words($section_description, $section_tooltip_description_length, '');
				}
			}
			$section_img = '';
			$section_img_id = '';
			if(isset($sections_arr[$i]['image']))
			{
				$img_url = $sections_arr[$i]['image']->getAttribute('src');
				$section_img_id = $this->find_img_id_by_url($img_url);
				$section_img = wp_get_attachment_image_src($section_img_id);
				$section_img = ($section_img ? $section_img[0] : $img_url);
			}

			$classes = $sections_arr[$i]['section']->getAttribute('class');
			$classes = preg_split('/\s+/', $classes);
			$section_class = 'pn-section-' . ($sections_count + $i);
			$section_class_found = false;
			for($j=0, $n=count($classes); $j<$n; $j++)
			{
				if(strpos($classes[$j], 'pn-section-')!==false)
				{
					if(!$section_class_found)
					{
						//check if post already has information about found section
						if($post_sections_count = count($post_sections_data))
						{
							for($k=0; $k<$post_sections_count; $k++)
							{
								if($post_sections_data[$k]['class']==$classes[$j])
								{
									$section_title = $post_sections_data[$k]['title'];
									$section_description = $post_sections_data[$k]['description'];
									$section_img = $post_sections_data[$k]['img'];
									$section_img_id = $post_sections_data[$k]['img_id'];
									break;
								}
							}
						}
						
						$classes[$j] =  $section_class;
						$section_class_found = true;
					}
					else
						unset($classes[$j]);
				}
			}
			
			if(!$section_class_found)
				$classes[] = $section_class;

			$sections_arr[$i]['section']->setAttribute('class', implode(' ', $classes));

			$this->sections[] = array(
				'title'			=> $section_title,
				'class'			=> $section_class,
				'description'	=> $section_description,
				'img'			=> $section_img,
				'img_id'		=> $section_img_id,
			);
		}

		$content_new = $document->saveHTML();

		//if modified content contains only white spaces then return original
		if(strlen(preg_replace('/\s+/', '', $content_new)))
			$content = $content_new;
		else
			return $content;
		
		return $this->extract_head_body_from_html_document($content);
//		return str_replace('”', '"', html_entity_decode($content));
	}
	
	
	/**
	 * DOMDocument::saveHTML() method will return HTML document along 
	 * with html/head/body tags, we just need to return the contents 
	 * of head and body tags
	 */
	function extract_head_body_from_html_document($html_document)
	{
		$head_start = array();
		preg_match("/<head[^>]*>/is", $html_document, $head_start, PREG_OFFSET_CAPTURE);
		$head_end = strpos($html_document, '</head>');
		$offset = $head_end[0][1]+strlen($head_end[0][0]);
		$lenght = $head_end-$offset;
		$head_content = substr($html_document, $offset, $lenght);
		
		$body_start = array();
		preg_match("/<body[^>]*>/is", $html_document, $body_start, PREG_OFFSET_CAPTURE);
		$body_end = strpos($html_document, '</body>');
		$offset = $body_start[0][1]+strlen($body_start[0][0]);
		$lenght = $body_end-$offset;
		$body_content = substr($html_document, $offset, $lenght);
		return $head_content . $body_content;;
	}
	
	
	function selector_to_array($selector)
	{
		$classes_id_arr = array();
		$this->strpos_classes_id($selector, $classes_id_arr);
		$parts = array();
		$dots_id_pos = array_keys($classes_id_arr);
		$dots_id_type = array_values($classes_id_arr);
		if($count=count($dots_id_pos))
		{
			for($i=0, $count=count($dots_id_pos); $i<$count; $i++)
			{
				if($i==0 && $dots_id_pos[$i]>0)
				{
					//get html tag
					$parts[] = array(
						'val'	=>	substr($selector, 0, $dots_id_pos[$i]),
						'type'	=>	'tag',
					);
				}

				$start_pos = $dots_id_pos[$i]+1;
				$length = isset($dots_id_pos[$i+1]) ? $dots_id_pos[$i+1]-$start_pos : strlen($selector);
				$parts[] = array(
					'val'	=>	substr($selector, $start_pos, $length),
					'type'	=>	$dots_id_type[$i]=='id' ? 'id' : 'class',
				);
			}
		}
		else
		{
			$parts[] = array(
				'val'	=>	$selector,
				'type'	=>	'tag',
			);
		}
		return $parts;
	}
	
	function strpos_classes_id($str, &$dots_id, $offset=0)
	{
		$dot_pos = strpos($str, '.', $offset);
		$id_pos = strpos($str, '#', $offset);
		if($dot_pos===false && $id_pos===false)
			return;
		if($dot_pos===false && $id_pos!==false)
		{
			$dots_id[$id_pos] = 'id';
			$this->strpos_classes_id($str, $dots_id, $id_pos+1);
		}
		if($dot_pos!==false && $id_pos===false)
		{
			$dots_id[$dot_pos] = 'class';
			$this->strpos_classes_id($str, $dots_id, $dot_pos+1);
		}
		if($dot_pos!==false && $id_pos!==false)
		{
			if($dot_pos < $id_pos)
			{
				$dots_id[$dot_pos] = 'class';
				$this->strpos_classes_id($str, $dots_id, $dot_pos+1);
			}
			else
			{
				$dots_id[$id_pos] = 'id';
				$this->strpos_classes_id($str, $dots_id, $id_pos+1);
			}
		}
	}
	
	function css_selector_to_xpath($css_selector)
	{
		$xpath_query = '';
		$selector_parts = preg_split('/\s+/', $css_selector);
		if(count($selector_parts))
		{
			$xpath_query .= '//';
			foreach($selector_parts as $selector_part)
			{
				$parts = $this->selector_to_array($selector_part);

				foreach($parts as $key=>$part)
				{
					if($key==0 && $part['type']=='tag')
						$xpath_query .= $part['val'];
					elseif($key==0 && $part['type']!='tag')
						$xpath_query .= '*';
					if($part['type']=='id')
						$xpath_query .= '[@id="' . $part['val'] . '"]';
//						$xpath_query .= '[contains(id, "' . $part['val'] . '")]';
					if($part['type']=='class')
						$xpath_query .= '[contains(concat(" ", normalize-space(@class), " "), " ' . $part['val'] . ' ")]';
//						$xpath_query .= '[contains(@class, "' . $part['val'] . '")]';
				}
				$xpath_query .= '//';
			}
			$xpath_query = rtrim($xpath_query, '//');
		}
		return $xpath_query;
	}
	
	/**
	 * Will search for sections images using the provided CSS selectors.
	 */
	function find_sections_titles($xpath, $section_title_selectors)
	{
		$xpath_query = '';
		if(count($section_title_selectors))
		{
			foreach($section_title_selectors as $section_title_selector)
			{
				$xpath_query .= $this->css_selector_to_xpath($section_title_selector);
				$xpath_query .= '|';
			}
			$xpath_query = rtrim($xpath_query, '|');
		}
		if(!$result = $xpath->query($xpath_query))
			return array();
		$sections_titles = array();
		for($i=0, $count=$result->length; $i<$count; $i++)
			$sections_titles[] = $result->item($i);
		return $sections_titles;
	}
	
	
	/**
	 * Will search for sections images using the provided CSS selectors.
	 */
	function find_sections_images($xpath, $section_img_selectors)
	{
		$xpath_query = '';
		if(count($section_img_selectors))
		{
			foreach($section_img_selectors as $section_img_selector)
			{
				$xpath_query .= $this->css_selector_to_xpath($section_img_selector);
				$xpath_query .= '|';
			}
			$xpath_query = rtrim($xpath_query, '|');
		}
		if(!$result = $xpath->query($xpath_query))
			return array();
		$sections_images = array();
		for($i=0, $count=$result->length; $i<$count; $i++)
			$sections_images[] = $result->item($i);
		return $sections_images;
	}
	
	function findDomNodeList($xpath, $selector)
	{
		$xpath_query = '';
		if(count($selector))
		{
			foreach($selector as $selector)
			{
				$xpath_query .= $this->css_selector_to_xpath($selector);
				$xpath_query .= '|';
			}
			$xpath_query = rtrim($xpath_query, '|');
		}
		if(!$result = $xpath->query($xpath_query))
			return array();
		$dom_node_list = array();
		for($i=0, $count=$result->length; $i<$count; $i++)
			$dom_node_list[] = $result->item($i);
		return $dom_node_list;
	}
	
	/**
	 * Will search for sections using the provided CSS selectors.
	 */
	function find_sections($xpath, $section_selectors)
	{
		$xpath_query = '';
		if(count($section_selectors))
		{
			foreach($section_selectors as $section_selector)
			{
				$xpath_query .= $this->css_selector_to_xpath($section_selector);
				$xpath_query .= '|';
			}
			$xpath_query = rtrim($xpath_query, '|');
		}
		if(!$result = $xpath->query($xpath_query))
			return array();
		$sections = array();
		for($i=0, $count=$result->length; $i<$count; $i++)
			$sections[] = $result->item($i);
		return $sections;
	}
	
	/**
	 * Will search for sections configured previously.
	 */
	function find_sections_old($xpath)
	{
		$result = $xpath->query('//*[contains(@class, "pn-section-")]');
		$sections_old = array();
		for($i=0, $count=$result->length; $i<$count; $i++)
			$sections_old[] = $result->item($i);
		return $sections_old;
	}
	
	function remove_unwanted_sections($sections, $sections_old)
	{
		//Explanation-1
		//we need to find old sections that aren't included in the new sections
		for($i=0, $count_i=count($sections_old); $i<$count_i; $i++)			
		{
			for($j=0, $count_j=count($sections); $j<$count_j; $j++)
			{
				if($sections_old[$i]===$sections[$j])
				{
					unset($sections_old[$i]);
					break;
				}
			}
		}
		//remove old sections that are unwanted
		if(count($sections_old))
		{
			foreach($sections_old as $section_old)
			{
				$css_classes = preg_split('/\s+/', $section_old->getAttribute('class'));
				//remove the pn-section-X class from the element
				for($j=0, $count_j=count($css_classes); $j<$count_j; $j++)
				{
					if(strpos($css_classes[$j], 'pn-section-') !== false)
						unset($css_classes[$j]);
				}
				$section_old->setAttribute('class', implode(' ', $css_classes));
			}
		}
	}
	
	function find_img_id_by_url($img_url)
	{
		global $wpdb;
		//fix for thumbnails
		$img_url = preg_replace('/-[\d]{1,5}x[\d]{1,5}/', '', $img_url);
		$query = $wpdb->prepare('SELECT ID FROM ' . $wpdb->posts . ' WHERE guid="%s";', $img_url);
		
		$result = $wpdb->get_col($query);
		return (isset($result[0]) ? $result[0] : false);
	}
	
}