<?php

class SiteContentNavigator
{
	public $pageSections;
	public $pnOptions;
	public $pnOptionsPost;
	public $navigationWidget;
	public $postId;
	public $postType;
	
	public function __construct()
	{
		//hooks and filters
		add_action('wp_loaded', array($this, 'init'), 10);
		add_action('wp_enqueue_scripts', array($this, 'load_scripts_styles'));
		add_action('admin_enqueue_scripts', array($this, 'admin_load_scripts_styles'));
		add_action('wp', array($this, 'set_post'));
		
		$pnOptions = new PNOptions();
		
		if($pnOptions->get_option('processing_method')=='output_buffering')
		{
			add_action('wp_head', array($this, 'buffer_start'), 1);
			add_action('wp_footer', array($this, 'buffer_end'), 10);
		}
		
		//Explanation-4:
		if($pnOptions->get_option('processing_method')=='filter_the_content')
			add_filter('the_content', array($this, 'search_sections'), 12);
		add_filter('the_content', array($this, 'post_ending'), 12);
		add_action('wp_footer', array($this, 'generate_html_code'), 100);
		add_action('widgets_init', array($this, 'register_widgets'));
		
		add_filter("plugin_action_links_" . PN_PLUGIN_BASENAME, array($this, 'documentation_link'));
		add_filter("plugin_action_links_" . PN_PLUGIN_BASENAME, array($this, 'settings_link'));
	}
	
	public function init()
	{
		$this->pnOptions = new PNOptions();
		$this->pnOptions->handle_wp();
		$this->pnOptionsPost = new PNOptionsPost();
		$this->pnOptionsPost->post_types = $this->pnOptions->get_option('post_types');
		$this->pnOptionsPost->handle_wp();
		$this->pageSections = new PageSections();
		$this->navigationWidget = array();
	}
	
	public function set_post($query)
	{
		if((int)$this->postId)
			return;
		$post_id = 0;
		$postType = 'page';
		if(is_home())
		{
			$post_id = get_option('page_for_posts');
			$postType = 'page';
		}
		elseif(is_category() || is_tag() || is_archive() || is_search() || is_404() || is_author())
		{
			$post_id = 0;
			$postType = 'page';
		}
		else
		{
			global $post;
			if(is_object($post) && property_exists($post, 'ID'))
			{
				$post_id = $post->ID;
				$postType = get_post_type();
			}
			else
			{
				$post_id = 0;
				$postType = 'page';
			}
		}
		
		$this->postType = $postType;
		$this->postId = $post_id;
		$this->pnOptions->set_post_id($post_id);
		$this->pageSections->pnOptions->set_post_id($post_id);
	}
	
	public function search_sections($content)
	{
		global $post;
		if(!$this->pnOptions->is_post_type_enabled($this->postType) || $post->ID!=$this->postId || $this->pnOptions->post_sections)
			return $content;
		
		return $this->pageSections->process_content($content);
	}
	
	public function generate_html_code()
	{
		if(!$this->pnOptions->is_post_type_enabled($this->postType))
			return;
			
		$this->include_js_vars();
		$this->include_custom_css();
		$this->include_navigation_bar();
		$this->include_navigation_widget();
	}
	
	public function include_js_vars()
	{
		//pass data to javascript
		$pn_config = array();
		if($this->pnOptions->post_sections!==false)
			$page_sections = $this->pnOptions->post_sections;
		else
			$page_sections = $this->pageSections->sections;
		$pn_config['sections'] = array_map(create_function('$v', 'return array("class" => $v["class"]);'), $page_sections);
		$pn_config['post_title'] = get_the_title($this->postId);
		$pn_config['overlap_site_content'] = $this->pnOptions->overlap_site_content;
		$pn_config['section_offset'] = $this->pnOptions->section_offset;
		$pn_config['progress_bar_algorithm'] = $this->pnOptions->progress_bar_algorithm;
		$pn_config['bar_ending'] = $this->pnOptions->bar_ending;
		$pn_config['sections_container_selectors'] = $this->pnOptions->sections_container_selectors;
		
		foreach($pn_config as $key=>$value)
		{
			if (!is_scalar($value))
				continue;
			$pn_config[$key] = html_entity_decode((string)$value, ENT_QUOTES, 'UTF-8');
		}

		$output = '';
		$output .= '
		<script type="text/javascript">
		pn_config = ' . wp_json_encode($pn_config) . ';
		</script>';
		echo $output;
	}
	
	public function include_custom_css()
	{
		echo include(PN_TEMPLATE_PATH . 'custom_css.php');
	}
	
	public function include_navigation_bar()
	{
		if(!$this->pnOptions->enable_navigation_bar)
			return;
		
		if($this->pnOptions->post_sections!==false)
			$page_sections = $this->pnOptions->post_sections;
		else
			$page_sections = $this->pageSections->sections;
		$section_count = count($page_sections);
		
		$custom_classes = array();
		if(!$this->pnOptions->enable_sections || !$section_count)
			$custom_classes[] = 'pn-no-sections';
		elseif($this->pnOptions->navigation_menu=='disabled')
			$custom_classes[] = 'pn-navigation-menu-disabled';
		elseif($this->pnOptions->navigation_menu=='visible_on_hover')
			$custom_classes[] = 'pn-navigation-menu-on-hover';
		elseif($this->pnOptions->navigation_menu=='always_visible')
			$custom_classes[] = 'pn-navigation-menu-always';
		$custom_classes[] = ($this->pnOptions->show_section_marks==='on_hover' ? 'pn-sections-marks-on-hover' : '');
		$custom_classes = implode(' ', $custom_classes);
		
		echo include(PN_TEMPLATE_PATH . 'navigation_bar.php');
	}
	
	public function include_navigation_widget()
	{
		if(count($this->navigationWidget))
		{
			extract($this->navigationWidget);
			if($this->pnOptions->post_sections!==false)
				$page_sections = $this->pnOptions->post_sections;
			else
				$page_sections = $this->pageSections->sections;
			echo include(PN_TEMPLATE_PATH . 'navigation_widget.php');
		}
	}
	
	public function load_scripts_styles()
	{
		if(!$this->pnOptions->is_post_type_enabled($this->postType))
			return;
		
		//public
		//load styles
		wp_enqueue_style('pn-style', PN_STYLE_URL . 'style.css', false, false);
		wp_enqueue_style('pn-responsive', PN_STYLE_URL . 'responsive.css', false, false);
		
		//load fonts
		if($this->pnOptions->header_font!='' && $this->pnOptions->header_font_custom=='')
			wp_enqueue_style('pn-font-header', '//fonts.googleapis.com/css?family=' . urlencode($this->pnOptions->header_font) . (is_array($this->pnOptions->header_font_subset) ? '&subset=' . implode(',', $this->pnOptions->header_font_subset) : ''));
		else
			wp_enqueue_style('pn-font-header', '//fonts.googleapis.com/css?family=Raleway:400,600');
		
		if($this->pnOptions->text_font!='' && $this->pnOptions->text_font_custom=='')
			wp_enqueue_style('pn-font-text', '//fonts.googleapis.com/css?family=' . urlencode($this->pnOptions->text_font) . (is_array($this->pnOptions->text_font_subset) ? '&subset=' . implode(',', $this->pnOptions->text_font_subset) : ''));
		else
			wp_enqueue_style('pn-font-text', '//fonts.googleapis.com/css?family=Open+Sans');
				
		//load scripts
		wp_enqueue_script('jquery');
		wp_enqueue_script('pn-script', PN_SCRIPT_URL . 'main.js', array('jquery'), false, true);
	}
	
	public function admin_load_scripts_styles()
	{
		//load styles
		wp_enqueue_style('pn-admin-style', PN_STYLE_URL . 'admin/style.css', false, false);
		wp_enqueue_style('pn-admin-colorpicker', PN_STYLE_URL . 'admin/colorpicker.css', false, false);

		//load scripts
		wp_enqueue_script('jquery');
		wp_enqueue_script('pn-admin-colorpicker', PN_SCRIPT_URL . 'admin/colorpicker.js', array('jquery'));
		wp_enqueue_script('pn-admin-script', PN_SCRIPT_URL . 'admin/main.js', array('jquery'), false, true);
	}
	
	function register_widgets()
	{
		return register_widget('PNNavigationWidget');
	}
	
	function buffer_start()
	{
		ob_start(array($this, 'handle_ob'));
	}
	
	function buffer_end()
	{
		ob_end_flush();
	}
	
	function handle_ob($content)
	{
		if(!$this->pnOptions->is_post_type_enabled($this->postType) || $this->pnOptions->post_sections)
			return $content;
		$before_body = '';
		preg_match('/<body[^>]*>/is', $content, $matches, PREG_OFFSET_CAPTURE);
		$body_offset = $matches[0][1]+strlen($matches[0][0]);
		$before_body = substr($content, 0, $body_offset);
		$body_content = substr($content, $body_offset);
		
		return $before_body . $this->pageSections->process_content($body_content);
	}
	
	function post_ending($content)
	{
		if(!$this->pnOptions->is_post_type_enabled($this->postType))
			return $content;
		
		global $post;
		return $content . '<div class="pn-post-ending"></div>';
	}
	
	function documentation_link($links)
	{
		$documentation_link = '<a href="' . plugins_url('documentation/index.html', PN_PLUGIN_BASENAME) . '" title="' . __('Documentation', 'site_content_navigator') . '">' . __('Documentation', 'site_content_navigator') . '</a>'; 
		array_unshift($links, $documentation_link); 
		return $links;
	}
	
	function settings_link($links)
	{
		$settings_link = '<a href="options-general.php?page=site-content-navigator.php" title="' . __('Settings', 'site_content_navigator') . '">' . __('Settings', 'site_content_navigator') . '</a>'; 
		array_unshift($links, $settings_link); 
		return $links;
	}
	
}
