"use strict";
jQuery(document).ready(function ($) {
	
	initPlugin();
	
	
	function initPlugin()
	{
		initColorpicker();

		//assign handlers
		$('#enable_sections').on('change', handlerGroupOptionChange);
		$('#progress_bar_algorithm').on('change', handlerProgressBarAlgorithmChange);
		$('#header_font').on('change', handlerHeaderFontChange)
		$('#text_font').on('change', handlerTextFontChange)
		$('#section_tooltip').on('change', handlerSubGroupOptionChange);
		$('.pn_post_option').on('change', handlerPostOptionChange);
		$('#search_sections').on('click', handlerSearchSections);
		$('#add_section').on('click', handlerAddSection);
		$(document).on('click', '.delete_section', handlerDeleteSection);
		$(document).on('click', '.pn_select_image', handlerSelectImage);
		$(document).on('paste change', 'input[name="section_img[]"]', handlerImgUrlChange);
	}
	
	function initColorpicker()
	{
		if($(".pn-color").length)
		{
			$(".pn-color").each(function() {
				var $this = $(this);
				$this.ColorPicker({
					onChange: function(hsb, hex, rgb, el) {
						$this.val(hex);
						$this.prev(".pn-color-preview").css("background-color", "#" + hex);				
					},
					onSubmit: function(hsb, hex, rgb, el){
						$(el).val(hex);
						$(el).ColorPickerHide();
					},
					onBeforeShow: function (){
						var color = (this.value!="" ? this.value : $(this).attr("data-default-color"));
						$(this).ColorPickerSetColor(color);
						$(this).prev(".pn-color-preview").css("background-color", color);				
					}
				}).on('keyup', function(event, param){
					$(this).ColorPickerSetColor(this.value);			
					var default_color = $(this).attr("data-default-color");
					$(this).prev(".pn-color-preview").css("background-color", (this.value!="none" ? (this.value!="" ? "#" + (typeof(param)=="undefined" ? $(".colorpicker:visible .colorpicker_hex input").val() : this.value) : (default_color!="transparent" ? "#" + default_color : default_color)) : "transparent"));
				});
			});

		}
	}
	
	function handlerHeaderFontChange(event)
	{
		var $this = $(this);
		if($this.val()!="")
		{
			var data = {
				action: 'pn_get_font_subsets',
				font: $this.val()
			}
			$.post(ajaxurl, data, function(data) {
				if(typeof(data.font_subsets)!='undefined')
				{
					$('#header_font_subset').html(data.font_subsets);
					$('.header_font_subset').removeClass('pn-hide');
				}
			},'json');
		}
		else
			$('.header_font_subset').addClass('pn-hide').find("option").remove();
	}
	
	function handlerTextFontChange(event)
	{
		var $this = $(this);
		if($this.val()!="")
		{;
			var data = {
				action: 'pn_get_font_subsets',
				font: $this.val()
			}
			$.post(ajaxurl, data, function(data) {
				if(typeof(data.font_subsets)!='undefined')
				{
					$('#text_font_subset').html(data.font_subsets);
					$('.text_font_subset').removeClass('pn-hide');
				}
			},'json');
		}
		else
			$('.text_font_subset').addClass('pn-hide').find("option").remove();
	}
	
	function handlerProgressBarAlgorithmChange(event)
	{
		var $this = $(this);
		if($this.val()!='algorithm_1')
			$('#progress_bar_color_2').closest('tr').removeClass('pn-hide');
		else
			$('#progress_bar_color_2').closest('tr').addClass('pn-hide');
	}
	
	function handlerGroupOptionChange(event)
	{
		var $this = $(this);
		if(parseInt($this.val()))
			$('.group_' + $this.attr('data-group')).removeClass('pn-hide');
		else
			$('.group_' + $this.attr('data-group')).addClass('pn-hide');
	}
	
	function handlerSubGroupOptionChange(event)
	{
		var $this = $(this);
		if(parseInt($this.val()))
			$('.group_' + $this.attr('data-group')).removeClass('pn-subhide');
		else
			$('.group_' + $this.attr('data-group')).addClass('pn-subhide');
	}
	
	function handlerImgUrlChange(event)
	{
		var $this = $(this);
		$this.next().val('');
	}
	
	function handlerPostOptionChange(event)
	{
		var $this = $(this);
		var postOption = $this.attr('data-post-option');
		var $postOption = $('.' + postOption);
		if($this.val()==0)
			$postOption.show();
		else
			$postOption.hide();
	}
	
	function handlerSelectImage(event)
	{
		event.preventDefault();
		var $this = $(this);
		wp.media.frames.selectImageFrame=wp.media(
		{
			multiple		:	false,
			library			: 
			{
			   type			:	'image',
			}
		});
		wp.media.frames.selectImageFrame.open();
		wp.media.frames.selectImageFrame.on('select',function()
		{
			var selection = wp.media.frames.selectImageFrame.state().get('selection');
			if(!selection) return;
			selection.each(function(attachment)
			{
				var img_url = attachment.attributes.url;
				var img_id = attachment.attributes.id;
				var $imgField = $('#' + $this.attr('data-img-field'));				
				var $imgIdField = $imgField.next();
				
				if($imgField.length)
					$imgField.val(img_url);
				if($imgIdField.length)
					$imgIdField.val(img_id);
			});
		});
	}
	
	function handlerDeleteSection(event)
	{
		event.preventDefault();
		
		var r = confirm("Delete section ?");
		if(r != true)
			return;

		//delete section
		var section_index = $(this).attr('data-section-index');
		var section_class = $('.post_section_' + section_index + ' input[name="section_class[]"]').val();
		var content = getWPEditorContent();
		content = content.replace(section_class, '');
		setWPEditorContent(content);		
		$('.post_section_' + section_index).remove();
	}
	
	function handlerAddSection(event)
	{
		event.preventDefault();
		var section_index = $(".pn-post_sections.pn-section-start").length;
		var data = {
			'action': 'pn_add_post_section',
			'section_index': section_index,
		};

		$.post(ajaxurl, data, function(response) {
			if(typeof(response.post_section_empty_html)!='undefined')
			{
				$('#pn-options-post .post_sections_default').after(response.post_section_empty_html);
			}
			
		}, 'json');
	}
	
	function handlerSearchSections(event)
	{
		event.preventDefault();
		
		var post_content,
			post_id,
			section_index;
		
		post_content = getWPEditorContent();
		post_id = $('#post_ID').val();
		section_index = $(".pn-post_sections.pn-section-start.manual").length;
		
		//prepare sections informations
		//include only sections that were added automatically
		var $sections_titles = $('.post_sections_settings:not(.manual) input[name="section_title[]"]');
		var $sections_class = $('.post_sections_settings:not(.manual) input[name="section_class[]"]');
		var $sections_img = $('.post_sections_settings:not(.manual) input[name="section_img[]"]');
		var $sections_img_id = $('.post_sections_settings:not(.manual) input[name="section_img_id[]"]');
		var $sections_description = $('.post_sections_settings:not(.manual) textarea[name="section_description[]"]');
		
		var post_sections = [];
		var i = 0;
		while($sections_titles.eq(i).length)
		{
			post_sections.push({
				'title'			: $sections_titles.eq(i).val(),
				'class'			: $sections_class.eq(i).val(),
				'img'			: $sections_img.eq(i).val(),
				'img_id'		: $sections_img_id.eq(i).val(),
				'description'	: $sections_description.eq(i).val(),				
			});
			i++;
		}
		
		//prepare options informations
		var post_options = {};
		if($('#section_selectors_default').val()==0)
			post_options.section_selectors = $('#section_selectors').val();
		else
			post_options.section_selectors_default = 1;
		if($('#section_title_selectors_default').val()==0)
			post_options.section_title_selectors = $('#section_title_selectors').val();
		else
			post_options.section_title_selectors_default = 1;
		if($('#section_img_selectors_default').val()==0)
			post_options.section_img_selectors = $('#section_img_selectors').val();
		else
			post_options.section_img_selectors_default = 1;
		
		var data = {
			'action': 'pn_find_post_sections',
			'section_index': section_index,
			'post_content': post_content,
			'post_id': post_id,
			'post_sections': post_sections,
			'post_options': post_options
		};

		$.post(ajaxurl, data, function(response) {
			if(typeof(response.post_content)!='undefined' && response.post_content.length>0)
			{
				setWPEditorContent(response.post_content);
			}
			
			if(typeof(response.page_sections_html)!='undefined' && response.page_sections_html.length>0)
			{
				$('#pn-options-post .post_sections_settings:not(.manual)').remove();
				
				if($('#pn-options-post .post_sections_settings:last').length)
					$('#pn-options-post .post_sections_settings:last').after(response.page_sections_html);
				else
					$('#pn-options-post .post_sections_default').after(response.page_sections_html);
			}
			
		}, 'json');
	}
	
	function getWPEditorContent()
	{
		return (tinyMCE.activeEditor!=null && !tinyMCE.activeEditor.isHidden() ? 
			tinyMCE.activeEditor.getContent() : $("#content").val());
	}
	function setWPEditorContent(content)
	{
		if(tinyMCE.activeEditor!=null && !tinyMCE.activeEditor.isHidden())
			tinyMCE.activeEditor.setContent(content);
		else
			$('#content').val(content);
	}
})