'use strict';
jQuery.extend(jQuery.easing,
{
	def: 'easeOutQuad',
	easeInOutCubic: function (x, t, b, c, d) {
        if ((t/=d/2) < 1) return c/2*t*t*t + b;
        return c/2*((t-=2)*t*t + 2) + b;
    },
});
jQuery(document).ready(function ($) {
	var barCompletePX,
		barInProgressPX,
		bodyMarginTop,
		calcBodyMarginTop,
		offsetTop,
		windowHeight,
		windowWidth,
		documentOffsetTop,
		widgetOffsetTop,
		widgetBottom,
		postHeight,
		$postEnding,
		sectionOffsetTopWindow,
		errorMargin;
	
	initPlugin();	
	function initPlugin()
	{
		initPostEnding();
		
		offsetTop = 0;
		windowWidth = $(window).width();
		windowHeight = $(window).height();
		postHeight = getPostHeight();
		documentOffsetTop = $(document).scrollTop();
		errorMargin = 3;
		sectionOffsetTopWindow = parseInt(pn_config.section_offset);
		widgetBottom = ($postEnding.length ? $postEnding.offset().top : 0);
		
		initBodyMarginTop();
		bodyMarginTop = calcBodyMarginTop();
		
		postHeight = getPostHeight();
		if(typeof(pn_config.sections)=='undefined')
			return;
		
		initSections();
		initNavSectionsList();
		initSectionMarks();
		initNavigationWidget();
		
		//assign event handlers
		$('#pn-navigation-bar, .pn-section-mark, .pn-nav-switch, .pn-navigation-info .pn-sections-list li, ul.navigation_widget_list li').on({
			mouseenter: handleDesktopHoverIn,
			mouseleave: handleDesktopHoverOut,
		});
		$(document).on('click touchstart', handleDocumentClickTouch);	
		$('#pn-navigation-bar').on('touchstart', handleNavBarTouch);
		$('.pn-section-mark').on('click touchstart', handleSectionMarkClickTouch);
		$('.pn-progress-bar-wrapper,.pn-clickable-area').on('click touchstart', handleProgressBarClickTouch);
		$('.pn-next,.pn-prev').on('click touchstart', handlePaginationClickTouch);
		$('.pn-nav-switch').on('click touchstart', handleNavSwitchClickTouch);
		$('ul.pn-sections-list li').on('click touchstart', handleNavSectionsClickTouch);
		$('ul.navigation_widget_list li').on('click touchstart', handleNavigationWidgetClickTouch);
		$(window).on('resize scroll orientationchange', handleScreenChange);
		$(window).on('orientationchange', setSectionsMenuHeight);
		
		setInterval(documentHeightChange, 200);
		
		calcProgress(true);
	}

	function calcProgress(force)
	{
		force = (typeof force !== 'undefined' ? force : false);
		
		if(!force)
		{
			//check if we need to re-calculate the progress bar or NavigationWidget
			if(windowWidth==$(window).width() && 
				windowHeight==$(window).height() && 
				postHeight==getPostHeight() &&
				documentOffsetTop==$(document).scrollTop())
				return;
		}
		
		windowWidth = $(window).width();
		windowHeight = $(window).height();
		postHeight = getPostHeight();
		documentOffsetTop = $(document).scrollTop();
		widgetBottom = ($postEnding.length ? $postEnding.offset().top : 0);
		bodyMarginTop = calcBodyMarginTop();
		
		if(pn_config.progress_bar_algorithm=='algorithm_1')
		{
			barInProgressPX = 0;
			barCompletePX = calcPosXAlg1(documentOffsetTop);
		}
		
		if(pn_config.progress_bar_algorithm=='algorithm_2')
		{
			barInProgressPX = calcPosXAlg2(windowHeight-sectionOffsetTopWindow-offsetTop);
			barCompletePX = calcPosXAlg2(documentOffsetTop);
		}
		
		setBodyMarginTop();
		
		//adjust position of widget and progress bar
		adjustOffsetTop();
		
		//handle sticky option
		navigationWidgetSticky();
		
		setProgressBar();
		
		if(typeof(pn_config.sections)=='undefined')
			return;
		
		calcSections();
		setNavSectionsList();
		setSectionsMarks();
		setNavigationWidget();
		setCurrentSection();
	}
	
	function calcSections()
	{
		var removeSections = [];
		pn_config.sections.forEach(function (el, index, array) {
			var $section = $('.' + el.class);
			var countRemaining = array.length-index-1;
			
			if($section.length)
			{
				el.visible = ($section.is(':visible') ? 1 : 0);
				
				if(pn_config.progress_bar_algorithm=='algorithm_1')
				{
					el.positionPX = calcPosXAlg1($section.offset().top-sectionOffsetTopWindow-offsetTop);
					if(el.positionPX>(windowWidth-8-(countRemaining*15)))
						el.positionPX = windowWidth-8-(countRemaining*15);
				}
				
				if(pn_config.progress_bar_algorithm=='algorithm_2')
				{
					el.positionPX = calcPosXAlg2($section.offset().top-sectionOffsetTopWindow-offsetTop);
				}
			}
			else
				removeSections.push(index);
		});
		removeDeadSections(removeSections);
	}

	function setProgressBar()
	{
		if(!$('#pn-navigation-bar').length)
			return;
		$('.pn-complete').css('width', barCompletePX + 'px');
		$('.pn-in-progress').css({
			'width' : barInProgressPX + 'px',
			'left' : barCompletePX + 'px'
		});
	}
	
	function setSectionsMarks()
	{
		if(!$('.pn-section-mark').length)
			return;
		pn_config.sections.forEach(function (el, index, array) {
			var $sectionMark = $('.' + el.class + '-mark');
			$sectionMark.css('left', el.positionPX + 'px');
			if(!el.visible)
			{
				$sectionMark.hide();
				return;
			}
			else
				$sectionMark.show();
			
			//determine section status: completed, in-progress, unseen
			if((el.positionPX-errorMargin)>(barInProgressPX+barCompletePX))
			{
				$sectionMark.removeClass('pn-section-in-progress pn-section-complete');
			}
			else if((el.positionPX-errorMargin)>barCompletePX)
			{
				$sectionMark.removeClass('pn-section-complete');
				$sectionMark.addClass('pn-section-in-progress');
			}
			else
			{
				$sectionMark.removeClass('pn-section-in-progress');
				$sectionMark.addClass('pn-section-complete');
			}
			
			//make sure that section tooltip doesn't go beyond the visible screen
			var $sectionInfo = $sectionMark.find('.pn-section-tooltip-wrapper');
			if($sectionMark.offset().left+parseInt($sectionInfo.css('width'))>windowWidth)
				$sectionInfo.addClass('pn-display-left');
			else
				$sectionInfo.removeClass('pn-display-left');
		});
	}
	
	function setNavSectionsList()
	{
		pn_config.sections.forEach(function (el, index, array) {
			var $sectionsListItem = $('.' + el.class + '-list');
			if(!el.visible)
			{
				$sectionsListItem.hide();
				return;
			}
			else
				$sectionsListItem.show();
		});
	}
	
	function setNavigationWidget()
	{
		var $sectionsWidget = $('ul.navigation_widget_list');
		if(!$sectionsWidget.length)
			return;
		
		pn_config.sections.forEach(function (el, index, array) {
			var $sectionsWidget = $('.' + el.class + '-widget'),
				nextSection = array[index+1],
				nextSectionPosPX,
				progress;
				
			if(!el.visible)
			{
				$sectionsWidget.hide();
				return;
			}
			else
				$sectionsWidget.show();
			
//			nextSectionPosPX = (typeof(nextSection)!='undefined' ? Math.round(nextSection.positionPX) : windowWidth);
			if(typeof(nextSection)!='undefined')
			{
				nextSectionPosPX = Math.round(nextSection.positionPX);
			}
			else
			{
				var temp;
				if(pn_config.progress_bar_algorithm=='algorithm_1')
					temp = ($postEnding.length ? $postEnding.offset().top : $(document).height());
				if(pn_config.progress_bar_algorithm=='algorithm_2')
					temp = ($postEnding.length ? $postEnding.offset().top : $(document).height())-sectionOffsetTopWindow-offsetTop;
				nextSectionPosPX = Math.round(windowWidth*temp/postHeight);
			}
			
			if((el.positionPX-errorMargin)>barCompletePX)
			{
				$sectionsWidget.removeClass('pn-active');
				$sectionsWidget.find('.section_progress').css('width', '0');
			}
			else if((el.positionPX-errorMargin)<=barCompletePX && barCompletePX<nextSectionPosPX)
			{
				progress = ((barCompletePX-el.positionPX)/(nextSectionPosPX-el.positionPX))*100;
				$sectionsWidget.addClass('pn-active');
				$sectionsWidget.find('.section_progress').css('width', progress + '%');
			}
			else
			{
				$sectionsWidget.addClass('pn-active');
				$sectionsWidget.find('.section_progress').css('width', '100%');
			}
		});
	}
	
	function navigationWidgetSticky()
	{
		var $sectionsWidget = $('ul.navigation_widget_list');
		if(!$sectionsWidget.length || $sectionsWidget.children('ul').hasClass('pn-no-sticky'))
			return;
				
		if($sectionsWidget.css('position')!='fixed')
			widgetOffsetTop = $sectionsWidget.offset().top;		
		var sectionsWidgetHeight = $sectionsWidget.height();
		var rowHeight = widgetBottom-widgetOffsetTop;
		
		if(window.innerwidth<=767)
		{
			//widget default position
			$sectionsWidget.css({
				'position': '',
				'top': '',
				'bottom': '',
				'width': '',
				'margin-top': '',
				'margin-bottom': '',
			});
			return;
		}
			
		if(sectionsWidgetHeight<windowHeight && 
			widgetOffsetTop-offsetTop-30<documentOffsetTop && 
			widgetOffsetTop-offsetTop-30+rowHeight-sectionsWidgetHeight>documentOffsetTop)
		{
			$sectionsWidget.css({
				'position': 'fixed',
				'top': (offsetTop+30)+'px',
				'bottom': 'auto',
				'width': $sectionsWidget.parent().css('width'),
				'margin-top': '0',
				'margin-bottom': '0',
			});
		}
		else if(sectionsWidgetHeight<windowHeight && 
				widgetOffsetTop-offsetTop-30+rowHeight-sectionsWidgetHeight<=documentOffsetTop && 
				(rowHeight-sectionsWidgetHeight>0))
		{
			$sectionsWidget.css({
				'position': 'fixed',
				'top': '',
				'bottom': (windowHeight-widgetBottom+documentOffsetTop)+'px',
				'width': $sectionsWidget.parent().css('width'),
				'margin-top': '0',
				'margin-bottom': '0',
			});
		}
		else if(sectionsWidgetHeight>=windowHeight && 
				widgetOffsetTop+30+sectionsWidgetHeight<documentOffsetTop+windowHeight && 
				widgetOffsetTop+30+rowHeight>documentOffsetTop+windowHeight)
		{
			$sectionsWidget.css({
				'position': 'fixed',
				'top': '',
				'bottom': '30px',
				'width': $sectionsWidget.parent().css('width'),
				'margin-top': '0',
				'margin-bottom': '0',
			});
		}
		else if(sectionsWidgetHeight>=windowHeight && 
				widgetOffsetTop+30+rowHeight<=documentOffsetTop+windowHeight && 
				(rowHeight-sectionsWidgetHeight>0))
		{
			$sectionsWidget.css({
				'position': 'fixed',
				'top': '',
				'bottom': (windowHeight-widgetBottom+documentOffsetTop)+'px',
				'width': $sectionsWidget.parent().css('width'),
				'margin-top': '0',
				'margin-bottom': '0',
			});
		}
		else
		{
			$sectionsWidget.css({
				'position': '',
				'top': '',
				'bottom': '',
				'width': '',
				'margin-top': '',
				'margin-bottom': '',
			});
		}
	}

	function adjustOffsetTop()
	{
		var $navigationBar = $('#pn-navigation-bar');
		var $sectionsWidget = $('ul.navigation_widget_list');
		
		var adminBarOffset = 0;

		if($('#wpadminbar').length)
		{
			if($('#wpadminbar').css('position')=='fixed')
				adminBarOffset = $('#wpadminbar').height();
			else if($('#wpadminbar').css('position')=='absolute')
			{
				adminBarOffset = ($('#wpadminbar').height()>documentOffsetTop ? $('#wpadminbar').height()-documentOffsetTop : 0);
			}
		}
		adminBarOffset = Math.round(adminBarOffset);
		
		if($navigationBar.length && $navigationBar.css('position')=='fixed')
		{
			$navigationBar.css('top', adminBarOffset + 'px');
			offsetTop = adminBarOffset + bodyMarginTop;
		}
		else
			offsetTop = adminBarOffset;
		offsetTop = Math.round(offsetTop);
		
		if($sectionsWidget.length && $sectionsWidget.css('position')=='fixed' && false)
		{
			if(typeof($sectionsWidget.attr('data-top'))=='undefined')
				$sectionsWidget.attr('data-top', parseInt($sectionsWidget.css('top')));
			$sectionsWidget.css('top', parseInt($sectionsWidget.attr('data-top')) + offsetTop + 'px');
		}
	}	
	
	function setCurrentSection()
	{
		var $lastSection = $('.pn-section-mark.pn-section-complete:last');
		$('.pn-section-current').html($lastSection.length ? $lastSection.attr('data-title') : pn_config.post_title);
		if($lastSection.length)
		{
			$('.pn-navigation-info .pn-sections-list li').removeClass('pn-active');
			$('.' + $('.pn-section-mark.pn-section-complete:last').attr('data-section-class') + '-list').addClass('pn-active');
		}
		else
			$('.pn-navigation-info .pn-sections-list li').removeClass('pn-active');
	}
	
	function getPostHeight()
	{
		var postHeight;
		if(pn_config.bar_ending=='end_of_content')
		{
			if(pn_config.progress_bar_algorithm=='algorithm_1')
				postHeight = ($postEnding.length ? $postEnding.offset().top : $(document).height());
			if(pn_config.progress_bar_algorithm=='algorithm_2')
				postHeight = ($postEnding.length ? $postEnding.offset().top : $(document).height())-sectionOffsetTopWindow-offsetTop;
		}
		if(pn_config.bar_ending=='end_of_page')
		{
			postHeight = $(document).height()-sectionOffsetTopWindow-offsetTop;
			if(pn_config.progress_bar_algorithm=='algorithm_1')
				postHeight = $(document).height();
			if(pn_config.progress_bar_algorithm=='algorithm_2')
				postHeight = $(document).height()-sectionOffsetTopWindow-offsetTop;
		}
		return postHeight;
	}
	
	//accordions/tabs/other may change document height
	//then we need to update the progress bar
	function documentHeightChange()
	{
		if(postHeight!=getPostHeight())
			$(window).trigger('resize');
	}
	
	function removeDeadSections(removeSections)
	{
		removeSections.sort(function(a,b){return b-a});
		removeSections.forEach(function (el, index, array) {
			$('.' + pn_config.sections[el].class + '-mark, .' + pn_config.sections[el].class + '-widget, .' + pn_config.sections[el].class + '-list').remove();
			pn_config.sections.splice(el,1);
		})
	}
	
	function initPostEnding()
	{
		var $sectionsContainer = $(pn_config.sections_container_selectors);
		if($sectionsContainer.length)
		{
			$('.pn-post-ending').remove();
			$sectionsContainer.eq(0).after('<div class="pn-post-ending"></div>');
			$postEnding = $('.pn-post-ending');
		}
		else
		{
			var $postEndingArr = $('.pn-post-ending');
			var count = $postEndingArr.length;
			if(count)
			{
				$postEnding = $postEndingArr.eq(count-1);
				if(count>1)
				{
					for(var i=0; i<(count-1); i++)
						$postEndingArr.eq(i).remove();
				}
			}
			else
			{
				$postEnding = {};
			}
		}
	}
	
	function initSections()
	{
		//remove sections that doesn't exist in the document
		var removeSections = [];
		pn_config.sections.forEach(function (el, index, array) {
			if(!$('.' + el.class).length)
				removeSections.push(index);
		});
		removeDeadSections(removeSections);
		
		//calculate sections position and sort by value
		pn_config.sections.forEach(function (el, index, array) {
			var $section = $('.' + el.class);
			var countRemaining = array.length-index-1;
			
			el.visible = ($section.is(':visible') ? 1 : 0);
			
			if(pn_config.progress_bar_algorithm=='algorithm_1')
			{
				el.positionPX = calcPosXAlg1($section.offset().top-sectionOffsetTopWindow-offsetTop);
				if(el.positionPX>(windowWidth-8-(countRemaining*15)))
					el.positionPX = windowWidth-8-(countRemaining*15);
			}
			
			if(pn_config.progress_bar_algorithm=='algorithm_2')
			{
				el.positionPX = calcPosXAlg2($section.offset().top-sectionOffsetTopWindow-offsetTop);
			}
		});
		pn_config.sections.sort(function(a, b) {
			return (a.positionPX>b.positionPX ? 1 : (a.positionPX<b.positionPX ? -1 : 0));
		});
	}
	
	function initNavSectionsList()
	{
		if($('ul.pn-sections-list li').length)
		{
			var sectionsList = [];
			pn_config.sections.forEach(function (el, index, array) {
				var $sectionList = $('.' + el.class + '-list');
				sectionsList.push($sectionList);
			});
			$('ul.pn-sections-list li').detach();
			$('ul.pn-sections-list').append(sectionsList);
		}
	}
	
	function initSectionMarks()
	{
		if($('.pn-section-mark').length)
		{
			var sectionsMarksArr = [];
			var max = pn_config.sections.length;
			pn_config.sections.forEach(function (el, index, array) {
				var $sectionMark = $('.' + el.class + '-mark');
				$sectionMark.find('.pn-pagination-index').html(index+1);
				$sectionMark.find('.pn-pagination-max').html(max);
				if(index==0)
				{
					$sectionMark.find('.pn-prev.pn-button').addClass('pn-inactive');
//					$sectionMark.find('.pn-prev.pn-button').remove();
				}
				else
				{
					var $sectionNext = $('.' + array[index-1].class + '-mark');
					$sectionMark.find('.pn-prev.pn-button').attr('data-section-class',$sectionNext.attr('data-section-class'));
				}

				if(index==(max-1))
				{
					$sectionMark.find('.pn-next.pn-button').addClass('pn-inactive');
//					$sectionMark.find('.pn-next.pn-button').remove();
				}
				else
				{
					var $sectionPrev = $('.' + array[index+1].class + '-mark');
					$sectionMark.find('.pn-next.pn-button').attr('data-section-class', $sectionPrev.attr('data-section-class'));
				}
				sectionsMarksArr.push($sectionMark);
			});
			$('.pn-section-mark').detach();
			$('#pn-navigation-bar .pn-progress-bar').append(sectionsMarksArr);
		}
	}
	
	function initNavigationWidget()
	{
		if($('#navigation_widget_wrapper').length && $('#navigation_widget_wrapper_source').length)
		{
			$('#navigation_widget_wrapper').html($('#navigation_widget_wrapper_source').html());
			$('#navigation_widget_wrapper_source').remove();
		}
		
		if($('ul.navigation_widget_list li').length)
		{
			var sectionsWidgetArr = [];
			pn_config.sections.forEach(function (el, index, array) {
				var $sectionsWidget = $('.' + el.class + '-widget');
				sectionsWidgetArr.push($sectionsWidget);
			});
			$('ul.navigation_widget_list li').detach();
			$('ul.navigation_widget_list').append(sectionsWidgetArr);
		}
	}
	
	function initBodyMarginTop()
	{
		calcBodyMarginTop = bodyMarginTopAlg0;
		if($('#pn-navigation-bar').length)
		{
			//case-1;case-4
			if(!$('#pn-navigation-bar').hasClass('pn-no-sections') &&  
				$('#pn-navigation-bar').hasClass('pn-navigation-menu-always'))
				calcBodyMarginTop = bodyMarginTopAlg1;
			
			//case-2;case-3
			if(!$('#pn-navigation-bar').hasClass('pn-no-sections') && 
				!$('#pn-navigation-bar').hasClass('pn-sections-marks-on-hover') && 
				!$('#pn-navigation-bar').hasClass('pn-navigation-menu-always'))
				calcBodyMarginTop = bodyMarginTopAlg2;
			
			//case-5-7
			if($('#pn-navigation-bar').hasClass('pn-no-sections') ||
				(!$('#pn-navigation-bar').hasClass('pn-no-sections') && 
				$('#pn-navigation-bar').hasClass('pn-sections-marks-on-hover') && 
					($('#pn-navigation-bar').hasClass('pn-navigation-menu-on-hover') || 
					$('#pn-navigation-bar').hasClass('pn-navigation-menu-disabled'))))
				calcBodyMarginTop = bodyMarginTopAlg3;
		}
	}
	
	
	function setBodyMarginTop()
	{
		if(!parseInt(pn_config.overlap_site_content))
		{
			if(!$('#pn-navigation-bar').hasClass('pn-hover'))
				$('body').css('margin-top', bodyMarginTop + 'px');
		}
		else
			$('body').css('margin-top', '');
	}
	
	function handleNavBarTouch(event)
	{
		var $this = $(this);
		$this.addClass('pn-hover');
		$('.pn-section-mark').removeClass('pn-hover');
		$('.pn-section-tooltip-wrapper').addClass('pn-hide');
		$('.pn-nav-switch').removeClass('pn-hover');
		if(event.type=='touchstart' && !$(event.target).hasClass('pn-sections-list-wrapper'))
			return false;
	}
	
	function handleDesktopHoverIn(event)
	{
		//windows phone will trigger hoverin, ignore it
		if(navigator.userAgent.match(/Windows Phone/i))
			return;
		
		var $this = $(this);		
		$this.addClass('pn-hover');
		
		if($this.hasClass('pn-section-mark'))
			$this.find('.pn-section-tooltip-wrapper').removeClass('pn-hide');
	}
	
	function handleDesktopHoverOut(event)
	{
		//windows phone will trigger hoverout, ignore it
		if(navigator.userAgent.match(/Windows Phone/i))
			return;
		
		var $this = $(this);
		
		if(!($this.is('#pn-navigation-bar') && $('.pn-nav-switch.open').length))
			$this.removeClass('pn-hover');
	}
	
	function handleDocumentClickTouch(event)
	{
		var $target = $(event.target);
		if(event.type=='click' && !$target.closest('#pn-navigation-bar').length)
		{
			$('#pn-navigation-bar').removeClass('pn-hover')
			$('.pn-section-mark').removeClass('pn-hover');
			$('.pn-section-tooltip-wrapper').addClass('pn-hide');
			closeSectionsMenu();
		}
		
		if(event.type=='touchstart' && !$(event.target).hasClass('pn-sections-list-wrapper'))
		{
			$('#pn-navigation-bar').removeClass('pn-hover')
			$('.pn-section-mark').removeClass('pn-hover');
			$('.pn-section-tooltip-wrapper').addClass('pn-hide');
			closeSectionsMenu();
		}
	}
	
	function handlePaginationClickTouch(event)
	{
		var $this = $(this);
		if($this.hasClass('pn-inactive'))
			return false;
		var targetSection = $this.attr('data-section-class');
		var $targetSection = $('.' + targetSection);
		scrollToSection($targetSection.offset().top);

		$('#pn-navigation-bar').removeClass('pn-hover');
		$('.pn-nav-switch').removeClass('pn-hover');
		closeSectionsMenu();
		$this.parents('.pn-section-mark').removeClass('pn-hover');
		$this.parents('.pn-section-tooltip-wrapper').addClass('pn-hide');

		if(event.type=='touchstart')
			return false;
	}
	
	function handleNavigationWidgetClickTouch(event)
	{
		var $this = $(this);
		var $section = $('.' + $this.attr('data-section-class'));
		scrollToSection($section.offset().top);
		return false;
	}
	
	function handleNavSectionsClickTouch(event)
	{
		var $this = $(this);
		var $section = $('.' + $this.attr('data-section-class'));
		scrollToSection($section.offset().top);
		
		$('#pn-navigation-bar').removeClass('pn-hover');
		if(event.type=='touchstart')
		{
			$('.pn-section-mark').removeClass('pn-hover');
			$('.pn-section-tooltip-wrapper').addClass('pn-hide');
		}
		closeSectionsMenu();

		if(event.type=='touchstart')
			return false;
	}
		
	function handleNavSwitchClickTouch(event)
	{
		var $this = $(this);
		if($this.hasClass('inactive'))
			return;
			
		if(event.type=='touchstart')
		{
			$('.pn-section-mark').removeClass('pn-hover');
			$('.pn-section-tooltip-wrapper').addClass('pn-hide');
			$this.addClass('pn-hover');
		}
		
		if($this.hasClass('open'))
			closeSectionsMenu();
		else
			openSectionsMenu();
		
		if(event.type=='touchstart')
			return false;
	}
	
	function handleProgressBarClickTouch(event)
	{
		var $this = $(this);
		var $target = $(event.target);
		//don't react if user clicked in section tooltip
		if($target.closest('.pn-section-mark').length)
			return;

		if(event.type=='touchstart')
		{
			$('#pn-navigation-bar').addClass('pn-hover');
			$('.pn-section-mark').removeClass('pn-hover');
			$('.pn-section-tooltip-wrapper').addClass('pn-hide');
			$('.pn-nav-switch').removeClass('pn-hover');
		}
		
		$('.pn-nav-switch').removeClass('pn-hover');
		closeSectionsMenu();

		var positionX = (event.type=="touchstart" ? event.originalEvent.touches[0].pageX : event.clientX);

		var offset;
		if(pn_config.progress_bar_algorithm=='algorithm_1')
			offset = calcPosYAlg1(positionX);
		if(pn_config.progress_bar_algorithm=='algorithm_2')
			offset = calcPosYAlg2(positionX);
		
		$('html, body').animate({scrollTop: offset}, 400);

		if(event.type=='touchstart')
			return false;
	}
	
	function handleSectionMarkClickTouch(event)
	{
		var $this = $(this);
		var $target = $(event.target);
		//react only if the user clicked on marker, not the tooltip
		if(!$target.is($this))
			return;

		if(event.type=='touchstart')
		{
			$('#pn-navigation-bar').addClass('pn-hover');
			$('.pn-section-mark').removeClass('pn-hover');
			$('.pn-section-tooltip-wrapper').addClass('pn-hide');
			$('.pn-nav-switch').removeClass('pn-hover');
			$this.addClass('pn-hover');
			$this.find('.pn-section-tooltip-wrapper').removeClass('pn-hide');
		}

		$('.pn-nav-switch').removeClass('pn-hover');
		var $section = $('.' + $this.attr('data-section-class'));
		scrollToSection($section.offset().top);

		if(event.type=='touchstart')
			return false;
	}
	
	function handleScreenChange()
	{
		//Explanation-5
		calcProgress();
	}
	
	function scrollToSection(sectionOffsetTop)
	{
		$('html, body').animate({scrollTop: sectionOffsetTop-sectionOffsetTopWindow-offsetTop}, 400);
	}
	
	function closeSectionsMenu()
	{
		$('.pn-nav-switch').removeClass('open').addClass('inactive');
		$('.pn-sections-list-wrapper').slideUp({
			'duration'		: 300,
			'easing'		: 'easeInOutCubic',
			'complete'		: setSectionsMenuHeight,
		});
	}
	
	function openSectionsMenu()
	{
		var $this = $('.pn-nav-switch');
		$this.addClass('open inactive');
		$('.pn-sections-list-wrapper').css({
			'display': 'block',
			'height': '',
		});
		var windowOffsetBottom = window.innerHeight+$(window).scrollTop();
		var sectionsMenuHeightAuto = $('.pn-sections-list-wrapper').outerHeight();
		var sectionsMenuOffsetBottom = sectionsMenuHeightAuto+$('.pn-sections-list-wrapper').offset().top;
		var diff = windowOffsetBottom-sectionsMenuOffsetBottom-20;
		var sectionsMenuHeightNew = sectionsMenuHeightAuto+diff;
		$('.pn-sections-list-wrapper').css({
			'height': '0',
			'overflow-y': '',
			'overflow-x': '',
		});

		if(diff>0)
		{
			$('.pn-sections-list-wrapper').stop(false, true).animate({
				height: sectionsMenuHeightAuto + 'px',
			},
			{
				'duration'		: 300,
				'easing'		: 'easeInOutCubic',
				'complete'		: function() {
					$this.removeClass('inactive');
				},
			});
		}
		else
		{
			$('.pn-sections-list-wrapper').css({
				'overflow-y': 'scroll',
				'overflow-x': 'hidden',
			});
			$('.pn-sections-list-wrapper').stop(false, true).animate({
				'height': sectionsMenuHeightNew + 'px',
			},
			{
				'duration'		: 300,
				'easing'		: 'easeInOutCubic',
				'complete'		: function() {
					$this.removeClass('inactive');
				},
			});
		}
	}
	
	function setSectionsMenuHeight()
	{
		$('.pn-nav-switch').removeClass('inactive');
		
		if($('.pn-sections-list-wrapper').is(':visible'))
		{
			var windowOffsetBottom = window.innerHeight+$(window).scrollTop();
			var sectionsMenuHeightAuto;
			var temp = $('.pn-sections-list-wrapper').outerHeight();
			sectionsMenuHeightAuto = $('.pn-sections-list-wrapper').css('height', '').outerHeight();
			$('.pn-sections-list-wrapper').css('height', temp + 'px');
			var sectionsMenuOffsetBottom = sectionsMenuHeightAuto+$('.pn-sections-list-wrapper').offset().top;
			var diff = windowOffsetBottom-sectionsMenuOffsetBottom-20;
			var sectionsMenuHeightNew = sectionsMenuHeightAuto+diff;

			if(diff>0)
			{
				$('.pn-sections-list-wrapper').css({
					'height': sectionsMenuHeightAuto + 'px',
					'overflow-y': '',
					'overflow-x': '',
				});
			}
			else if(diff<0 && (sectionsMenuHeightNew!=$('.pn-sections-list-wrapper').outerHeight()))
			{
				$('.pn-sections-list-wrapper').css({
					'height': sectionsMenuHeightNew + 'px',
					'overflow-y': 'scroll',
					'overflow-x': 'hidden',
				});
			}
		}
		else
		{
			$('.pn-sections-list-wrapper').css({
				'height': '',
				'overflow-y': '',
				'overflow-x': '',
			});
		}
	}
	
//	function calcBodyMarginTop()
//	{
//		var value = 0;
//		if($('#pn-navigation-bar').length)
//		{
//			//case-1;case-4
//			if(!$('#pn-navigation-bar').hasClass('pn-no-sections') &&  
//				$('#pn-navigation-bar').hasClass('pn-navigation-menu-always'))
//			{
//				console.log('case 1 or 4');
//				value = parseInt($('.pn-navigation-info').css('padding-top'))+
//					$('.pn-navigation-info .pn-section-current').height()+
//					$('.pn-navigation-info .pn-spacing').outerHeight();
//					
//				if($('#pn-navigation-bar').hasClass('pn-sections-marks-on-hover'))
//					value += $('#pn-navigation-bar .pn-progress-bar-wrapper .pn-progress-bar').height()+
//						parseInt($('#pn-navigation-bar .pn-progress-bar-wrapper').css('padding-bottom'));
//				else
//					value += $('#pn-navigation-bar .pn-progress-bar-wrapper').outerHeight();
//			}
//			
//			//case-2;case-3
//			if(!$('#pn-navigation-bar').hasClass('pn-no-sections') && 
//				!$('#pn-navigation-bar').hasClass('pn-sections-marks-on-hover') && 
//				!$('#pn-navigation-bar').hasClass('pn-navigation-menu-always'))
//			{
//				console.log('case 2 or 3');
//				value = $('#pn-navigation-bar .pn-progress-bar-wrapper').height()+
//					parseInt($('#pn-navigation-bar .pn-progress-bar-wrapper').css('padding-top'));
//			}
//			
//			//case-5-7
//			if($('#pn-navigation-bar').hasClass('pn-no-sections') ||
//				(!$('#pn-navigation-bar').hasClass('pn-no-sections') && 
//				$('#pn-navigation-bar').hasClass('pn-sections-marks-on-hover') && 
//					($('#pn-navigation-bar').hasClass('pn-navigation-menu-on-hover') || 
//					$('#pn-navigation-bar').hasClass('pn-navigation-menu-on-disabled'))))
//			{
//				console.log('case 5, 6, ... 12');
//				value = $('#pn-navigation-bar .pn-progress-bar-wrapper').height();
//			}
//		}
//		
//		return Math.round(value);
//	}
	
	function calcPosXAlg1($val)
	{
		return Math.round(windowWidth*$val/(postHeight-windowHeight));
	}
	
	function calcPosXAlg2($val)
	{
		return Math.round(windowWidth*$val/postHeight);
	}
	
	function calcPosYAlg1($val)
	{
		return Math.round(($val/windowWidth)*(postHeight-windowHeight));
	}
	
	function calcPosYAlg2($val)
	{
		return Math.round(($val/windowWidth)*postHeight);
	}
	
	//case-0
	function bodyMarginTopAlg0()
	{
		return 0;
	}
	
	//case-1;case-4
	function bodyMarginTopAlg1()
	{
		var value;
		value = parseInt($('.pn-navigation-info').css('padding-top'))+
		$('.pn-navigation-info .pn-section-current').height()+
		$('.pn-navigation-info .pn-spacing').outerHeight();

		if($('#pn-navigation-bar').hasClass('pn-sections-marks-on-hover'))
			value += $('#pn-navigation-bar .pn-progress-bar-wrapper .pn-progress-bar').height()+
				parseInt($('#pn-navigation-bar .pn-progress-bar-wrapper').css('padding-bottom'));
		else
			value += $('#pn-navigation-bar .pn-progress-bar-wrapper').outerHeight();
		return Math.round(value);
	}
	
	//case-2;case-3
	function bodyMarginTopAlg2()
	{
		var value;
		value = $('#pn-navigation-bar .pn-progress-bar-wrapper').height()+parseInt($('#pn-navigation-bar .pn-progress-bar-wrapper').css('padding-top'));
		return Math.round(value);
	}
	
	//case-5-7
	function bodyMarginTopAlg3()
	{
		var value;
		value = $('#pn-navigation-bar .pn-progress-bar-wrapper').height();
		return Math.round(value);
	}
	
});